/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.activities;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.support.design.widget.TabLayout;
import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;
import android.view.MenuItem;
import android.view.ViewGroup;
import android.widget.Toast;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.response.AccountInformation;
import com.thingspace.media.R;
import com.thingspace.media.dialog.SortDialog;
import com.thingspace.media.listeners.SortOptionChangeListener;
import com.thingspace.media.listeners.ThingspaceResponseListener;
import com.thingspace.media.tasks.GetAccountInfoTask;
import com.thingspace.media.tasks.HubbleMediaTask;
import com.thingspace.media.tasks.RetrieveContactsTask;
import com.thingspace.media.tasks.RetrieveFullViewTask;
import com.thingspace.media.tasks.UploadTask;
import com.thingspace.media.utils.PathFinder;

import java.text.DecimalFormat;
import java.util.ArrayList;

import javax.net.ssl.HttpsURLConnection;

/**
 * Base activity performs authorization by taking App Key, Secret and Callback URL. If authorized, VerizonCloudActivity is called which displays all the Photos/Videos, Music and Contacts.
 * This activity also gives options to Refresh, Search, Upload, Sort, Account Information and Logout which are performed by making SDK API calls in the onOptionsItemSelected method.
 * This Activity calls RetrieveFullViewTask, GetAccountInfoTask, UploadTask which in turn calls Cloud SDK APIs to perform corresponding functionalities.
 */
public class BaseActivity extends AppCompatActivity implements ThingspaceResponseListener, SortOptionChangeListener {

    private static final int FILE_SELECT_CODE = 0;

    protected CloudAPI cloudAPI;
    private boolean authenticationStarted;
    private boolean fullviewRunning;
    private ProgressDialog progressDialog;
    private ArrayList<SortOptionChangeListener> sortOptionChangeListeners = new ArrayList<>();


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        /*
         * Initialize Cloud SDK API object using the Application Key, Secret and Callback URL that can be obtained from Key Management tab in ThingSpace portal.
         * Application Key, Secret and Callback URL needs to be updated in the build.gradle file
         */
        cloudAPI = new CloudAPI(getString(R.string.APP_ID), getString(R.string.APP_SECRET), getString(R.string.CALLBACK_URL));
    }

    @Override
    protected void onResume() {
        super.onResume();
        /*
         * Start authorization if user is not yet authorized.
         * Upon authorization, Login/consent screen will be displayed asking user to grant verizon cloud access to the application.
         * After authorization is completed, onResume of this activity will be called.
         */
        if (!cloudAPI.hasAuthorization(this)) {
            if (!authenticationStarted) {
                startAuthorization();
            } else {
                finish();
            }
        } else if (authenticationStarted) {
            authenticationStarted = false;
            onAuthorizationCompleted();
        }
    }

    /**
     * Start user authentication and application authorization with Thingspace API.
     */
    private void startAuthorization() {
        authenticationStarted = true;
        // Start the remote authentication
        cloudAPI.authorize(this);
    }

    protected void onAuthorizationCompleted() {

    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        switch (item.getItemId()) {
            case android.R.id.home:
                onBackPressed();
                return true;
            case R.id.action_revoke_authorization:
                logOut();
                return true;
            case R.id.action_fullview:
                /**
                 * Retrieve information about all the files stored under user's cloud storage account by making a Cloud SDK API call
                 */
                if (!fullviewRunning) {
                    fullviewRunning = true;
                    RetrieveFullViewTask task = new RetrieveFullViewTask(this, this, cloudAPI, false);
                    task.execute();
                    retrieveContacts();
                }
                return true;
            case R.id.action_search:
                /**
                 * Search with options such as Advanced filters and Custom Search in Files and Folders stored under user's cloud by calling SearchActivity
                 */
                Intent intent = new Intent(this, SearchActivity.class);
                startActivity(intent);
                return true;
            case R.id.action_account_info:
                /**
                 * Retrieves the account information of how much memory is used of available storage by making a Cloud SDK API call
                 */
                GetAccountInfoTask getAccountInfoTask = new GetAccountInfoTask(this, this, cloudAPI);
                getAccountInfoTask.execute();
                showProgressDialog(getString(R.string.action_retrieving_info));
                return true;
            case R.id.action_sort:
                /**
                 * Display a dialog to sort Photo/Video/Music albums/files by date or name.
                 */
                SortDialog dialog = new SortDialog();
                dialog.show(getSupportFragmentManager(), SortDialog.TAG);
                return true;
            case R.id.action_upload:
                /**
                 * Displays the file chooser to select a file from the gallery to upload it to cloud.
                 */
                showFileChooser();
                return true;
            case R.id.action_shares:
                /**
                 * Opens an activity to list all shares.
                 */
                startActivity(new Intent(this, SharesListActivity.class));
        }

        return super.onOptionsItemSelected(item);
    }

    private void logOut() {
        /**
         * Logs out the user by clearing the token and cache.
         */
        cloudAPI.logout(this);
        Intent intent = new Intent(this, VerizonCloudActivity.class);
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK);
        intent.putExtra(VerizonCloudActivity.EXTRA_EXIT, true);
        startActivity(intent);
    }

    @Override
    public void onResponse(int taskCode, int responseCode, Object response) {

        if (responseCode == HttpsURLConnection.HTTP_UNAUTHORIZED) {
            /*
             * 401 - Authorization has been revoked.
             * Display a message "Session no longer valid" if authorization is revoked
             */
            Toast.makeText(this, getString(R.string.session_invalid), Toast.LENGTH_SHORT).show();
            onAuthorizationInvalid();
            startAuthorization();
            return;
        }
        /**
         * This switch case handles Cloud SDK API response
         */
        switch (taskCode) {
            case HubbleMediaTask.TASK_FULLVIEW:
                fullviewRunning = false;
                break;
            case HubbleMediaTask.TASK_DOWNLOAD:
                if (response instanceof Boolean) {
                    Toast.makeText(this, getString(((Boolean) response).booleanValue() ? R.string.file_downloaded : R.string.file_not_downloaded), Toast.LENGTH_SHORT).show();
                }
                break;
            case HubbleMediaTask.TASK_UPLOAD:
                if (response instanceof Boolean) {
                    Toast.makeText(this, getString(((Boolean) response).booleanValue() ? R.string.file_uploaded : R.string.file_not_uploaded), Toast.LENGTH_SHORT).show();
                }
                break;
            case HubbleMediaTask.TASK_ACCOUNT_INFO:
                if (response == null) {
                    Toast.makeText(this, "No new files added", Toast.LENGTH_SHORT).show();
                } else if (response instanceof AccountInformation) {
                    /**
                     * Displays Alert Dialog of how much memory is used of available storage
                     */

                    AccountInformation accountInformation = (AccountInformation) response;
                    dismissProgressDialog();
                    String alertMessage = "Used " + getFileSize(accountInformation.getUsedStorage()) + " of " + getFileSize(accountInformation.getAvailableStorage());
                    AlertDialog storagePopup = new AlertDialog.Builder(this)
                            .setTitle(getString(R.string.action_account_info))
                            .setCancelable(false)
                            .setMessage(alertMessage)
                            .setPositiveButton("Ok", null)
                            .create();
                    storagePopup.show();
                }
        }
    }

    protected void retrieveContacts() {
        /**
         * Retrieve all contacts stored under user's cloud storage account by making Cloud SDK API call.
         */
        new RetrieveContactsTask(this, this, cloudAPI).execute();
    }

    protected void onAuthorizationInvalid() {
        cloudAPI.logout(this);
    }

    /**
     * Get File size in terms of Byte, KiloByte, MegaByte, GigaByte and TeraByte
     */
    private String getFileSize(long size) {

        if (size <= 0)
            return "0";
        final String[] units = new String[]{"B", "KB", "MB", "GB", "TB"};
        int digitGroups = (int) (Math.log10(size) / Math.log10(1024));
        return new DecimalFormat("#.##").format(size / Math.pow(1024, digitGroups)) + " " + units[digitGroups];
    }

    public void registerSortOptionChangeListener(SortOptionChangeListener sortOptionChangeListener) {
        sortOptionChangeListeners.add(sortOptionChangeListener);
    }

    public void unRegisterSortOptionChangeListener(SortOptionChangeListener sortOptionChangeListener) {
        sortOptionChangeListeners.remove(sortOptionChangeListener);
    }

    /**
     * When sort option is changed, get the new option to sort accordingly
     */
    @Override
    public void onSortOptionChange() {
        for (int i = 0; i < sortOptionChangeListeners.size(); i++) {
            sortOptionChangeListeners.get(i).onSortOptionChange();
        }
    }

    /**
     * Opens the file chooser from which a file can be chosen to upload it to cloud.
     */
    private void showFileChooser() {
        Intent intent = new Intent(Intent.ACTION_GET_CONTENT);
        intent.setType("*/*");
        intent.addCategory(Intent.CATEGORY_OPENABLE);

        try {
            startActivityForResult(Intent.createChooser(intent, "Select a File to Upload"), FILE_SELECT_CODE);
        } catch (android.content.ActivityNotFoundException ex) {
            Toast.makeText(this, "Please install a File Manager.", Toast.LENGTH_SHORT).show();
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        switch (requestCode) {
            case FILE_SELECT_CODE:
                if (resultCode == RESULT_OK) {
                    // Get the Uri of the selected file
                    Uri uri = data.getData();
                    // Get the path
                    String path = PathFinder.getPath(this, uri);
                    if (TextUtils.isEmpty(path)) {
                        Toast.makeText(this, R.string.error_selecting_file, Toast.LENGTH_SHORT).show();
                        break;
                    }
                    String parentPath = path.substring(0, path.lastIndexOf("/"));
                    String fileName = path.substring(path.lastIndexOf("/") + 1);
                    /**
                     * Upload a file to the specified path in the cloud by making a CLoud SDK API call.
                     */
                    new UploadTask(this, this, "/VZMOBILE", fileName, parentPath, cloudAPI).execute();
                }
                break;
        }
        super.onActivityResult(requestCode, resultCode, data);
    }

    protected void showProgressDialog(String text) {
        progressDialog = ProgressDialog.show(this, "", text, true);
        progressDialog.setCancelable(true);
    }

    protected void dismissProgressDialog() {
        progressDialog.dismiss();
    }

    /**
     * Class to manage Tabs
     */
    protected class TabRunnable implements Runnable {

        private TabLayout tabLayout;
        private Context context;

        public TabRunnable(Context context, TabLayout tabLayout) {
            this.context = context;
            this.tabLayout = tabLayout;
        }

        @Override
        public void run() {
            if (tabLayout.getWidth() < context.getResources().getDisplayMetrics().widthPixels) {
                tabLayout.setTabMode(TabLayout.MODE_FIXED);
                ViewGroup.LayoutParams mParams = tabLayout.getLayoutParams();
                mParams.width = ViewGroup.LayoutParams.MATCH_PARENT;
                tabLayout.setLayoutParams(mParams);

            } else {
                tabLayout.setTabMode(TabLayout.MODE_SCROLLABLE);
            }
        }
    }
}
