/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.activities;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Environment;
import android.support.v7.widget.Toolbar;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.GestureDetector;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.WindowManager;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.Toast;

import com.thingspace.cloud.sdk.list.ItemList;
import com.thingspace.cloud.sdk.list.item.Item;
import com.thingspace.cloud.sdk.list.item.Photo;
import com.thingspace.cloud.sdk.list.item.PlaylistItem;
import com.thingspace.cloud.sdk.list.item.SystemAttributes;
import com.thingspace.media.R;
import com.thingspace.media.cache.ImageCache;
import com.thingspace.media.cache.ImageFetcher;
import com.thingspace.media.dialog.FileInfoDialog;
import com.thingspace.media.tasks.GetFileTask;

import java.io.File;
import java.io.FileOutputStream;

/**
 * PhotoActivity is called when a photo is selected to view it in full screen.
 * This Activity calls GetFileTask which in turn makes a CLoud SDK API call to get full file from cloud.
 */
public class PhotoActivity extends BaseActivity {

    public static final String EXTRA_ITEM_NUMBER = "extra_item_number";

    private static ImageFetcher staticImageFetcher;
    private static ItemList tempList;
    private ImageView fullPhoto;
    private ProgressBar progressBar;
    private String name;
    private String parentPath;
    private String path;
    private String mimeType;
    private String capturedDate;
    private String createdDate;
    private String width;
    private String height;
    private String size;
    private String contentToken;
    private ItemList itemList;
    private int itemNumber;

    private GetFileTask getFileTask;

    public static void setItemList(ItemList itemList) {
        tempList = itemList;
    }

    private static DisplayMetrics getDisplayMetrics(Context context) {
        DisplayMetrics displayMetrics = new DisplayMetrics();
        WindowManager wm = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
        wm.getDefaultDisplay().getMetrics(displayMetrics);
        return displayMetrics;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_photo);

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        fullPhoto = (ImageView) findViewById(R.id.imageFullPhoto);
        final GestureDetector gestureDetector = new GestureDetector(this, new GestureListener());
        fullPhoto.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(final View view, final MotionEvent event) {
                gestureDetector.onTouchEvent(event);
                return true;
            }
        });

        progressBar = (ProgressBar) findViewById(R.id.progressBarPhoto);
        itemNumber = getIntent().getIntExtra(EXTRA_ITEM_NUMBER, 0);

        itemList = tempList;
        tempList = null;

        loadFullPhoto(itemList.getItem(itemNumber));
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_photo, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        switch (item.getItemId()) {
            case R.id.action_info:
                /**
                 * Display information about the selected photo.
                 */
                FileInfoDialog fileInfoDialog = new FileInfoDialog();
                Bundle bundle = new Bundle();
                bundle.putString(FileInfoDialog.FILE_MIME, mimeType);
                bundle.putString(FileInfoDialog.FILE_DATE, capturedDate);
                bundle.putString(FileInfoDialog.FILE_WIDTH, width);
                bundle.putString(FileInfoDialog.FILE_HEIGHT, height);
                bundle.putString(FileInfoDialog.FILE_SIZE, size);
                bundle.putString(FileInfoDialog.FILE_NAME, name);
                fileInfoDialog.setArguments(bundle);
                fileInfoDialog.show(getFragmentManager(), FileInfoDialog.TAG);
                return true;
            case R.id.action_download:
                /**
                 * Download the photo to SD card.
                 */
                saveImageToSD();
                return true;
        }

        return super.onOptionsItemSelected(item);
    }

    /**
     * Retrieve full photo along with photo information from item object.
     * @param item
     */
    private void loadFullPhoto(Item item) {
        if (item instanceof Photo) {
            Photo photo = (Photo) item;
            name = photo.getName();
            parentPath = photo.getParentPath();
            mimeType = photo.getMimeType();
            size = String.valueOf(photo.getSize());
            contentToken = photo.getContentToken();
            createdDate = photo.getVersionCreated();
            width = String.valueOf(photo.getWidth());
            height = String.valueOf(photo.getHeight());
            capturedDate = photo.getCapturedDate();
        } else {
            SystemAttributes systemAttributes = (SystemAttributes) item;
            name = systemAttributes.getName();
            parentPath = systemAttributes.getParentPath();
            mimeType = systemAttributes.getMimeType();
            size = String.valueOf(systemAttributes.getSize());
            contentToken = systemAttributes.getContentToken();
            createdDate = systemAttributes.getVersionCreated();
            width = String.valueOf(systemAttributes.getWidth());
            height = String.valueOf(systemAttributes.getHeight());

            if (systemAttributes instanceof PlaylistItem) {
                path = ((PlaylistItem) systemAttributes).getPath();
                capturedDate = systemAttributes.getVersionCreated();
            } else {
                capturedDate = systemAttributes.getCapturedDate();
            }
        }

        getImageFetcher().loadImageFromCache(contentToken, fullPhoto);
        progressBar.setVisibility(View.VISIBLE);

        /**
         * If getFileTask is in process then cancel the task.
         */
        if (getFileTask != null && !getFileTask.getStatus().equals(AsyncTask.Status.FINISHED)) {
            getFileTask.cancel(true);
        }

        /**
         * GetFileTask makes a CLoud SDK API call to get full file from cloud.
         */
        getFileTask = new GetFileTask(this, this, parentPath, name, cloudAPI, fullPhoto, progressBar);
        getFileTask.execute();
    }

    /**
     * Retrieves the photo bytes from imageview bitmap and write it to the storage.
     */
    private void saveImageToSD() {
        File sdcard = Environment.getExternalStorageDirectory();

        File folder = new File(sdcard.getAbsoluteFile(), "VZDemoApp");
        folder.mkdir();
        File file = new File(folder.getAbsoluteFile(), name);
        if (file.exists()) {
            Toast.makeText(this, getString(R.string.download_exist), Toast.LENGTH_SHORT).show();
            return;
        }

        try {
            FileOutputStream out = new FileOutputStream(file);
            ((BitmapDrawable) fullPhoto.getDrawable()).getBitmap().compress(Bitmap.CompressFormat.JPEG, 100, out);
            out.flush();
            out.close();

            Toast.makeText(this, getString(R.string.download_completed), Toast.LENGTH_SHORT).show();
        } catch (Exception e) {
            Log.e(getClass().getSimpleName(), "Exception downloading the photo: " + e);
        }
    }

    private void moveNext() {
        if (itemList == null) {
            return;
        }

        try {
            Item item = itemList.next();
            String mimeType;
            if (item instanceof Photo) {
                mimeType = ((Photo) item).getMimeType();
            } else {
                mimeType = ((SystemAttributes) item).getMimeType();
            }

            if (mimeType != null && mimeType.indexOf("image") != -1) {
                loadFullPhoto(item);
            } else {
                moveNext();
            }
        } catch (Exception e) {
            Log.e(getClass().getSimpleName(), "Exception getting next content token:" + e);
        }
    }

    private void movePrevious() {
        if (itemList == null) {
            return;
        }

        try {
            Item item = itemList.prev();
            String mimeType;
            if (item instanceof Photo) {
                mimeType = ((Photo) item).getMimeType();
            } else {
                mimeType = ((SystemAttributes) item).getMimeType();
            }

            if (mimeType != null && mimeType.indexOf("image") != -1) {
                loadFullPhoto(item);
            } else {
                movePrevious();
            }
        } catch (Exception e) {
            Log.e(getClass().getSimpleName(), "Exception getting previous content token:" + e);
        }
    }

    public ImageFetcher getImageFetcher() {
        if (staticImageFetcher == null) {
            ImageCache.ImageCacheParams cacheParams = new ImageCache.ImageCacheParams(this, "thumbs");
            cacheParams.setMemCacheSizePercent(this, 0.5f);

            staticImageFetcher = new ImageFetcher(this, getResources().getDimensionPixelSize(R.dimen.photo_item_size));
            staticImageFetcher.setLoadingImage(R.drawable.thumb_placeholder);
            staticImageFetcher.addImageCache(cacheParams);
        }
        return staticImageFetcher;
    }

    /**
     * If gesture is done from Right to Left, move to the next item else if it is from Left to Right, move to the previous item.
     */
    private class GestureListener extends GestureDetector.SimpleOnGestureListener {
        @Override
        public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX, float velocityY) {
            if (e1.getX() - e2.getX() > (getDisplayMetrics(PhotoActivity.this).widthPixels) / 3) {
                // Right to Left
                moveNext();
            } else if (e2.getX() - e1.getX() > (getDisplayMetrics(PhotoActivity.this).widthPixels) / 3) {
                // Left to Right
                movePrevious();
            }
            return false;
        }
    }
}
