/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.fragments;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;

import com.thingspace.cloud.sdk.enums.PlaylistType;
import com.thingspace.cloud.sdk.enums.SortType;
import com.thingspace.cloud.sdk.list.ItemList;
import com.thingspace.cloud.sdk.list.item.Item;
import com.thingspace.cloud.sdk.list.item.Playlist;
import com.thingspace.media.R;
import com.thingspace.media.activities.AlbumsFilesActivity;
import com.thingspace.media.activities.PlaylistItemsActivity;
import com.thingspace.media.activities.SearchResultActivity;
import com.thingspace.media.adapters.OnBindViewInterface;
import com.thingspace.media.adapters.RecyclerViewListAdapter;
import com.thingspace.media.adapters.viewholders.RecyclerViewListHolder;
import com.thingspace.media.cache.SortPreference;
import com.thingspace.media.dialog.CreatePlaylistDialog;
import com.thingspace.media.tasks.playlist.DeletePlaylistTask;
import com.thingspace.media.tasks.HubbleMediaTask;
import com.thingspace.media.tasks.playlist.RetrieveAudioPlaylistsTask;
import com.thingspace.media.tasks.playlist.UpdatePlaylistDefinitionTask;

import java.util.ArrayList;

/**
 * MusicPlaylistFragment is called when "Playlist" Tab is selected in Music which retrieves and displays all Playlists stored in the cloud.
 * This Fragment calls RetrieveAudioPlaylistsTask which in turn makes an API call to retrieve all Playlists stored in the user's cloud storage account.
 */
public class MusicPlaylistFragment extends BaseFragment implements OnBindViewInterface {

    private static final int ACTION_UPDATE = 0;
    private static final int ACTION_UPDATE_DEFINITION = 1;
    private static final int ACTION_DELETE = 2;
    private ItemList<Playlist> playlistItemList;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = super.onCreateView(inflater, container, savedInstanceState);
        FloatingActionButton fab = (FloatingActionButton) view.findViewById(R.id.fab);
        fab.setVisibility(View.VISIBLE);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                CreatePlaylistDialog createPlaylistDialog = new CreatePlaylistDialog();
                Bundle bundle = new Bundle();
                bundle.putInt(CreatePlaylistDialog.EXTRA_PLAYLIST_TYPE, CreatePlaylistDialog.TYPE_MUSIC);
                bundle.putString(CreatePlaylistDialog.EXTRA_PLAYLIST_ACTION, CreatePlaylistDialog.ACTION_PLAYLIST_CREATE);
                createPlaylistDialog.setArguments(bundle);
                createPlaylistDialog.setTargetFragment(MusicPlaylistFragment.this, 111);
                CreatePlaylistDialog.setCloudAPI(cloudAPI);
                createPlaylistDialog.show(getFragmentManager(), "Create_playlist");
            }
        });

        return view;
    }

    public void executeQuery() {
        SortType sortType = SortType.NAME.setOrder(SortType.SortOrder.ASC);
        if (SortPreference.getSortPreference(getActivity()).equalsIgnoreCase(getActivity().getString(R.string.sort_date))) {
            sortType = SortType.CREATION_DATE.setOrder(SortType.SortOrder.ASC);
        }

        /**
         * RetrieveAudioPlaylistsTask makes a CLoud SDK API call to retrieve all audio playlists stored under user's cloud storage account.
         */
        RetrieveAudioPlaylistsTask retrievePlaylistsTask = new RetrieveAudioPlaylistsTask(getActivity(), this, cloudAPI, sortType);
        retrievePlaylistsTask.execute();
        showProgressDialog(getString(R.string.loading));
    }

    protected int getLayoutType() {
        return LAYOUT_TYPE_LIST;
    }

    protected RecyclerView.Adapter getRecyclerViewAdapter(ItemList itemList) {
        return new RecyclerViewListAdapter(R.layout.list_item, itemList, this, this);
    }

    @Override
    public void onBindView(RecyclerView.ViewHolder holder, Item item) {
        ((RecyclerViewListHolder) holder).textView.setText(((Playlist) item).getName());
    }

    /**
     * When a playlist is clicked, AlbumsFilesActivity is used to display contents/files from it.
     * @param view
     * @param position
     */
    @Override
    public void onListItemClicked(View view, int position) {
        AlbumsFilesActivity.setPlaylist(playlistItemList.getItem(position));
        Intent intent = new Intent(getActivity(), AlbumsFilesActivity.class);
        intent.putExtra(PlaylistItemsActivity.EXTRA_NAME, playlistItemList.getItem(position).getName());
        intent.putExtra(AlbumsFilesActivity.EXTRA_SOURCE_TYPE, AlbumsFilesActivity.TYPE_PLAYLIST);
        intent.putExtra(AlbumsFilesActivity.EXTRA_CONTENT_TYPE, AlbumsFilesActivity.CONTENT_TYPE_MUSIC);
        startActivity(intent);
    }

    @Override
    public void onListItemLongClicked(View view, final int position) {
        AlertDialog.Builder operationDialog = new AlertDialog.Builder(getActivity());
        final ArrayAdapter<String> arrayAdapter = new ArrayAdapter<>(getActivity(), R.layout.list_item);

        arrayAdapter.add(getString(R.string.action_update));
        arrayAdapter.add(getString(R.string.update_definition));
        arrayAdapter.add(getString(R.string.action_delete));

        operationDialog.setAdapter(arrayAdapter, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                switch (which) {
                    /**
                     * Update the Playlist name.
                     */
                    case ACTION_UPDATE:
                    /**
                     * Updates the playlist definition and gives an option to select new files which replaces the old files of the playlist.
                     */
                    case ACTION_UPDATE_DEFINITION:
                        CreatePlaylistDialog updatePlaylistDialog = new CreatePlaylistDialog();
                        Bundle bundle = new Bundle();
                        bundle.putInt(CreatePlaylistDialog.EXTRA_PLAYLIST_TYPE, CreatePlaylistDialog.TYPE_MUSIC);
                        bundle.putString(CreatePlaylistDialog.EXTRA_PLAYLIST_ACTION, which == ACTION_UPDATE ?
                                CreatePlaylistDialog.ACTION_PLAYLIST_UDPATE : CreatePlaylistDialog.ACTION_PLAYLIST_UDPATE_DEFINITION);
                        bundle.putString(CreatePlaylistDialog.EXTRA_PLAYLIST_NAME, playlistItemList.getItem(position).getName());
                        bundle.putString(CreatePlaylistDialog.EXTRA_PLAYLIST_SELECTED_TYPE, playlistItemList.getItem(position).getType());
                        bundle.putString(CreatePlaylistDialog.EXTRA_PLAYLIST_UID, playlistItemList.getItem(position).getUid());
                        updatePlaylistDialog.setArguments(bundle);
                        updatePlaylistDialog.setTargetFragment(MusicPlaylistFragment.this, 111);
                        CreatePlaylistDialog.setCloudAPI(cloudAPI);
                        updatePlaylistDialog.show(getFragmentManager(), "Update_playlist");
                        break;
                    /**
                     * Delete an existing playlist.
                     */
                    case ACTION_DELETE:
                        AlertDialog.Builder deletePopup = new AlertDialog.Builder(getActivity());
                        deletePopup.setMessage(R.string.delete_operation)
                                .setPositiveButton(getString(R.string.action_yes), new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        showProgressDialog(getString(R.string.loading));
                                        /**
                                         * DeletePlaylistTask makes a CLoud SDK API call to delete the selected Playlist.
                                         */
                                        DeletePlaylistTask task = new DeletePlaylistTask(getActivity(), MusicPlaylistFragment.this, cloudAPI, playlistItemList.getItem(position).getUid());
                                        task.execute();
                                    }
                                }).setNegativeButton(getString(R.string.action_no), null).create();
                        deletePopup.show();
                        break;
                    default:
                        break;

                }
            }
        });

        operationDialog.setTitle(getString(R.string.file_actions));
        operationDialog.create();
        operationDialog.show();
    }

    @Override
    public void onResponse(int taskCode, int errorCode, Object response) {
        dismissProgressDialog();
        if (taskCode == HubbleMediaTask.TASK_AUDIO_PLAYLIST) {
            if (response == null || !(response instanceof ItemList)) {
                return;
            }

            emptyView.setVisibility(View.GONE);
            playlistItemList = (ItemList<Playlist>) response;
            recyclerView.setAdapter(getRecyclerViewAdapter(playlistItemList));

            return;
        } else if ((taskCode == HubbleMediaTask.TASK_DELETE_PLAYLIST && (boolean) response) || taskCode == HubbleMediaTask.TASK_UPDATE_PLAYLIST_DEFINITION) {
            executeQuery();
        }
        super.onResponse(taskCode, errorCode, response);
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (data != null && data.hasExtra(SearchResultActivity.RESULT_SELECTED_PATHS)
                && data.hasExtra(SearchResultActivity.EXTRA_PLAYLIST_TYPE)) {

            /**
             * Receives the list of selected files to replace the files in the existing playlist.
             */
            ArrayList<String> paths = data.getStringArrayListExtra(SearchResultActivity.RESULT_SELECTED_PATHS);
            int contentType = data.getIntExtra(SearchResultActivity.EXTRA_PLAYLIST_TYPE, 1);
            String name = data.getStringExtra(SearchResultActivity.EXTRA_NAME);
            String uid = data.getStringExtra(SearchResultActivity.EXTRA_UID);

            /**
             * UpdatePlaylistDefinitionTask makes a cloud SDK API call to update the file name along with option to replace the files in the playlist selected.
             */
            UpdatePlaylistDefinitionTask updatePlaylistDefinitionTask = new UpdatePlaylistDefinitionTask(getActivity(), MusicPlaylistFragment.this,
                    cloudAPI, uid, name, paths, getPlaylistType(contentType));
            updatePlaylistDefinitionTask.execute();
        }
    }

    public PlaylistType getPlaylistType(int contentType) {
        switch (contentType) {
            case PlaylistItemsActivity.CONTENT_TYPE_IMAGE:
                return PlaylistType.IMAGE;
            case PlaylistItemsActivity.CONTENT_TYPE_IMAGE_VIDEO:
                return PlaylistType.IMAGE_VIDEO;
            case PlaylistItemsActivity.CONTENT_TYPE_VIDEO:
                return PlaylistType.VIDEO;
            case PlaylistItemsActivity.CONTENT_TYPE_MUSIC:
                return PlaylistType.MUSIC;
            default:
                return PlaylistType.IMAGE;
        }
    }
}
